@@ -1,17 +1,21 @@
+//go:build !remote
+
 package libpod
 
 import (
 	"bytes"
 	"errors"
 	"fmt"
+	"io/fs"
 	"net"
+	"os"
 	"strconv"
 	"strings"
 	"sync"
 	"time"
 
 	"github.com/containers/common/libnetwork/types"
-	"github.com/containers/podman/v4/libpod/define"
+	"github.com/containers/podman/v5/libpod/define"
 	"github.com/sirupsen/logrus"
 	bolt "go.etcd.io/bbolt"
 )
@@ -69,12 +73,26 @@ type BoltState struct {
 
 // NewBoltState creates a new bolt-backed state database
 func NewBoltState(path string, runtime *Runtime) (State, error) {
+	logrus.Info("Using boltdb as database backend")
 	state := new(BoltState)
 	state.dbPath = path
 	state.runtime = runtime
 
 	logrus.Debugf("Initializing boltdb state at %s", path)
 
+	// BoltDB is deprecated and, as of Podman 5.0, we no longer allow the
+	// creation of new Bolt states.
+	// If the DB does not already exist, error out.
+	// To continue testing in CI, allow creation iff an undocumented env
+	// var is set.
+	if os.Getenv("CI_DESIRED_DATABASE") != "boltdb" {
+		if _, err := os.Stat(path); err != nil && errors.Is(err, fs.ErrNotExist) {
+			return nil, fmt.Errorf("the BoltDB backend has been deprecated, no new BoltDB databases can be created: %w", define.ErrInvalidArg)
+		}
+	} else {
+		logrus.Debugf("Allowing deprecated database backend due to CI_DESIRED_DATABASE.")
+	}
+
 	db, err := bolt.Open(path, 0600, nil)
 	if err != nil {
 		return nil, fmt.Errorf("opening database %s: %w", path, err)
